"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class WalletPaymentOptionContextTypedDict(TypedDict):
    merchant_name: str
    supported_schemes: List[str]


class WalletPaymentOptionContext(BaseModel):
    merchant_name: str

    supported_schemes: List[str]
