"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from gr4vy.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, TypedDict


class PaymentMethodStoredCardTypedDict(TypedDict):
    id: str
    r"""The ID of the stored card to use."""
    method: Literal["id"]
    r"""Set to `id` to use a stored card."""


class PaymentMethodStoredCard(BaseModel):
    id: str
    r"""The ID of the stored card to use."""

    METHOD: Annotated[
        Annotated[Optional[Literal["id"]], AfterValidator(validate_const("id"))],
        pydantic.Field(alias="method"),
    ] = "id"
    r"""Set to `id` to use a stored card."""
